// Copyright (c) 2009 Tournier Guillaume
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Require Prototype 1.6+

if (typeof Prototype == 'undefined')
  alert('Calendar Error:\n- Prototype not found.\n- Make sure that your layout includes prototype.js *before* it includes calendar.js.');

Object.extend(Date, {
	firstDayOfWeek: 0,
	NOW: 'now', TODAY: 'today', AM_PM: ['AM', 'PM'],
	MONTHS: $w('January February March April May June July August September October November December'),
	SHORT_MONTHS: $w('Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec'),
	DAYS: $w('Sunday Monday Tuesday Wednesday Thursday Friday Saturday'),
	SHORT_DAYS: $w('Sun Mon Tue Wed Thu Fri Sat'),
  
	truncate_days: function(size) {
		return this.DAYS.map(function(d) { return d.substr(0, size || 1).capitalize(); });
	},
	_METHODS: { y: 'FullYear', yy: 'ShortYear', M: 'Month', d: 'Date', D: 'YearDate',
	E: 'Day', H: 'Hours', K: 'Hours', h: 'hours', k: 'kours', m: 'Minutes',
	s: 'Seconds', S: 'Milliseconds', a: 'AMPM', w: 'WeekInYear', W: 'WeekInMonth' },
  _MONTH_DAYS: [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334]
});

Object.extend(Date.prototype, {
	add: function(symbol, n) {
		var method = Date._METHODS[symbol];
		this['set' + method](this['get' + method].call(this) + n);
		return this;
	},
	set: function(symbol, n) {;
		this['set' + Date._METHODS[symbol]]('M' != symbol ? n : n - 1);
		return this;
	},
	get: function(symbol) {
		var value = this['get' + Date._METHODS[symbol]]();
		return ('M' != symbol ? value : value + 1);
	},
	clone: function() {
		return new Date(this.getTime());
	},
	isAfter: function(date) {
		  return this.getTime() > date.getTime();		
	},
	isBefore: function(date) {
		  return this.getTime() < date.getTime();		
	},
	resetTime: function() {
		return this.set('H', 0).set('m', 0).set('s', 0).set('S', 0);
	},
	sameDay: function(d) {
		return this.getDate() == d.getDate() && this.getMonth() == d.getMonth() &&
		  this.getFullYear() == d.getFullYear();
	},
	isWeekEnd: function() {
		return (6 + this.getDay()) % 7 > 4;
	},
	getWeekDay: function() {
		return (7 + this.getDay() - Date.firstDayOfWeek) % 7;
	},
	setkours: function(h) { this.setHours(24 == h ? 0 : h) },
	sethours: function(h) { this.setHours(12 == h ? 0 : h); },
	setDay: function(day) {
		this.setDate(this.getDate() + (7 - this.getDay() + day) % 7);
	},
	getCentury: function() {
		return (this.getFullYear() / 100).floor();
	},
	getShortYear: function() {
		return this.getFullYear() % 100;
	},  
	setShortYear: function(year) {
		var now = new Date(), year = new Number(year),
		  add = (now.getShortYear() + 21 < year ? - 1 : 0);
		this.setFullYear((now.getCentury() + add) * 100 + year);
	},
	getAMPM: function() {
		return Date.AM_PM[this.getHours() > 11 ? 1 : 0];
	},
	setAMPM: function(marker) {
		var m = this.getAMPM();
		if (m != marker.toLowerCase())
				this.add('H', Date.AM_PM[1] == m ? -12 : 12); 
	},
	isLeapYear: function() {
		var y = this.getFullYear();
		return y % 4 == 0 && y % 100 != 0 || y % 400 == 0;
	},
	getYearDate: function() {
		var m = this.getMonth();
		return Date._MONTH_DAYS[m] + this.getDate() + 
		  (m > 1 && this.isLeapYear() ? 1 : 0);
	},
	setYearDate: function(day) {
		this.setMonth(0); this.setDate(day);
	},
	setWeekInYear: function(week) {
		var day = (8 + this.getWeekDay() - this.getYearDate() % 7) % 7,
				date = (1 == week ? 1 : (1 - day + (week - 1) * 7));
		this.set('M', 1).setDate(date);
	},
	setWeekInMonth: function(week) {
		var day = (8 + this.getWeekDay() - this.getDate() % 7) % 7;
		this.setDate(1 == week ? 1 : (1 - day + (week - 1) * 7));
	},
	getWeekInMonth: function() {
		var d = this.getDate(), day = (9 + this.getWeekDay() - d % 7) % 7;
		d -= 8 - (0 == day ? 7 : day);
		return (1 + (d / 7).floor()) + (d % 7 != 0 ? 1 : 0);
	},
	getWeekInYear: function() {
		var d = this.getYearDate(), wd = this.getWeekDay(), day = (9 + wd - d % 7) % 7;
		d -= 8 - (0 == day ? 7 : day);
		var value = (1 + (d / 7).floor()) + (d % 7 != 0 ? 1 : 0);
		if (53 > value) return value;
		return this.getDate() + 6 - wd <= 31 ? value : 1;
	}
});


DateFormat = function(pattern) {
	var regFTokens = /('(?:[^']|'')*')|(([ydDMHhKkmsSaEwW])\3*)|([^\2])/g,
		regPTokens = /('(?:[^']|'')*')|(([ydDMHhKkmsSaEwW])+)|([^\2])/g,
		regSymbol = /([ydDMHhKkmsSaE])\1*/g, regNumber = /^(\s*[0-9]+).*$/,
		regYY = /^yy?$/, regMe = /^[ME]{3,}$/,
		regAmPm = new RegExp(Date.AM_PM.join('|')),
		regTrema = /^'(.*)'$/, regDtrema = /''/g, fTokens, pTokens,
		validRanges = $H({ M: [1, 12], H: [0, 23], k: [1, 24],
			K: [0, 11], h: [1, 12], m: [0, 59], S:[0, 999] });
	
	setPattern(pattern);
	
	function lz(str, n) {
		return '0'.times(n - ('' + str).length) + str;
	}
	
	function removeComments(str) {
		return str.replace(regTrema, '$1').replace(regDtrema, "'");
	}
	
	function names(type, len) {
		return Date[(len > 3 ? '' : 'SHORT_') + type];
	}
	
	function check(h, d) {
		return validRanges.inject(false, function(acc, pair) {
			var key = pair[0], value = pair[1];
			return acc || (h[key] && (h[key] < value[0] || h[key] > value[1]));
		}) || ['d', 'y', 'yy', 'a', 'D', 'E', 'w', 'W'].inject(false, function(acc, key) {
			return acc || (h[key] && h[key] != d.get(key));
		});
	}
	
	function parseElement(str, h, token, reg) {
		var symbol = (token.match(regYY) ? 'yy' : token.charAt(0)), len = token.length;
		if (!Date._METHODS[symbol]) {
			token = removeComments(token);
			if (token != str.substring(0, token.length))
				throw new Error('Unmatch: ' + str);
		} else if ('a' == symbol) {
			h['a'] = (token = str.substring(0, 2));
			if (!(token).match(regAmPm))
				throw new Error('Unmatch: ' + str);
		} else if (token.match(regMe)) {
			var m = 'M' == symbol;
			names(m ? 'MONTHS' : 'DAYS', len).each(function(value, i) {
				if (value != str.substring(0, value.length)) return;
				h[symbol] = m ? i + 1 : i;
				token = value;
				throw $break;
			});
		} else {
			h[symbol] = (token = str.replace(reg, '$1'));
		}
		return str.substring(token.length);
	}


	function format(date) {
		return (!fTokens || !date) ? '' : fTokens.inject('', function(acc, token) {
			var len = token.length, symbol = (token.match(regYY) ? 'yy' : token.charAt(0));
			if (!Date._METHODS[symbol]) return acc + removeComments(token);
			switch(symbol) {
				case 'a': return acc + date.get('a');
				case 'K': return acc + lz(date.get('H') % 12, len);
				case 'k': case 'h': var h = date.get('H'); h = (0 == h ? 24 : h);
					return acc + lz((h > 12 && 'h' == symbol ? h - 12 : h), len);
				case 'E': return acc + names('DAYS', len)[date.getDay()];
				case 'M': if (len > 2) return acc + names('MONTHS', len)[date.getMonth()];
				default: return acc + lz(date.get(symbol), len);
			}
		});
	}
  
	function parse(str) {
		try {
			var reg = new RegExp(), d = new Date(0).set('H', 0), h = {}, tmp;
			pTokens.each(function(token) {
				if ("'" != token.charAt(0) && (tmp = token.match(regSymbol)) && tmp.length > 1) {
					token = tmp.pop();
					for (var i = 0; i < tmp.length; ++i)
						str = parseElement(str, h, tmp[i], reg.compile('^([0-9]{1,' + tmp[i].length +'}).*$'));
				}
				str = parseElement(str, h, token, regNumber);
			});
			['yy', 'y', 'M', 'w', 'W', 'd', 'D', 'E', 'k', 'H', 'h', 'K', 'm', 's', 'S', 'a'].each(function(key) {
				if ('a' == key && !h['h'] && !h['K']) return;
				if (h[key]) d.set(key, h[key]);
			});
			if (check(h, d)) throw new Error('Invalid value');
			return d;
		} catch(e) { return null; }
	}
	
	function setPattern(pattern) {
		fTokens = pattern.match(regFTokens);
		pTokens = pattern.match(regPTokens);
	}
	
	return { format: format, parse: parse, setPattern: setPattern };
};

DateFormat.DEFAULT_PATTERN = 'M/d/yyyy';


Css = (function() {
	var head = $(document.getElementsByTagName('head')[0]),
		style = { bottom: undefined, top: undefined };
	
	function addStyleSheet(href, media) {
		head.insert('<link rel="stylesheet" type="text/css" href="' +
			href + '" media="' + (media || 'screen') + '">');
	}
	
	function addRules(rules, media, position) {
		position = 'top' == position ? 'top' : 'bottom';
		if (!style[position]) {
			style[position] = new Element('style', { type: 'text/css', media: media || 'screen' });
			head.insert('top' != position ? { bottom: style[position] } : { top: style[position] });
		}
		if (style[position].styleSheet)
			style[position].styleSheet.cssText += rules + '\n'; // IE sucks...
		else
			style[position].insert(rules + '\n');
	}
	
	return { addStyleSheet: addStyleSheet, addRules: addRules };
})();



Calendar = function(target, options) {
	// Set default css
	Calendar.addDefaultCss();
	
	// Set today + target
	var today = new Date(), target = $(target);
	if (!target) { throw new Error('Calendar: invalid target "' + target + '"'); }
	
	// Options
	options = options || {};
	options = Object.extend({
		pattern: options.pattern || DateFormat.DEFAULT_PATTERN,
		startYear: (today.getFullYear() - 1),
		yearRange: 10,
		startDate: null,
		endDate: null,
		popupBy: null,
		panel: 'normal',
		weeks: true,
		extraPanel: false,
		display: true,
		displayPattern: null,
		minutesRange: 5,
		embedded: false,
		fixed: false,
		strictMonth: false,
		monthNames: 'full',
		dayNames: 2,
		readonly: false
	}, options);
	options.popupBy = $(options.popupBy) || target;
	if (!options.displayPattern)
		options.displayPattern = options.pattern;
	
	var startDate, endDate, minYear, maxYear, time = false, amPm = false, h = false,
		events = {}, date, selected, height, dHeight, now, inputType, disabled,
		df = Calendar.dateFormat,visible, values, linkTo = [], linkFunc = [],
		id = 'jsCalendar_' + (++Calendar.id);

	setDefaultStartEndDates();
	setDefaultDate();
	setSelectedValue();
	
	minYear = startDate.resetTime().getFullYear();
	maxYear = endDate.resetTime().getFullYear();
	
	var sym = options.pattern
		.match(/('(?:[^']|'')*')|(([ydDMHhKkmsSaEwW])\3*)|([^\2])/g)
		.map(function(t) { return t.charAt(0); });
	time = ['m', 'H', 'K', 'h', 'k'].any(function(s) { return sym.include(s); });
	amPm = sym.include('a');
	h = sym.include('h');
	if (time) options.extraPanel = true;
	
	if (options.minutesRange)
		options.minutesRange = parseInt(options.minutesRange, 10);
	if (options.readonly) {
		target.writeAttribute('readonly', true);
		target.style.cursor = 'pointer';
	}
	
	// Build calendar
	var html =  '<table border="0">' + buildTHead() + buildTFoot() +
			'<tbody>' + buildContent() + '</tbody>' + '</table>',
		iframe, cal = new Element('div', { id: id, className: 'calendar',
			style: (options.embedded ? '' : 'display: none; position: absolute; z-index: 42;')});

	if (!options.embedded)
		document.body.appendChild(cal.update(html));
	else
		target.insert({ before: cal.update(html) });
	
	// Draw an iframe behind the calendar. IE Sucks...
	if (Prototype.Browser.IE && (!Calendar.iframe || options.embedded)) {
		if (!options.embedded || (options.embedded && 'absolute' == target.style.position)) {
			iframe = new Element('iframe', { frameBorder: 'no',
				style: 'position: absolute; z-index: 0; display: none;' });
			document.body.appendChild(iframe);
			if (!options.embedded) Calendar.iframe = iframe;
			else iframe.show();
		}
	}
	
	var table = cal.down('table'), tbody = table.down('tbody'),
		monthElement = cal.down('.months') || {}, yearElement = cal.down('.years') || {},
		panel, displayPanel, hourElement, minuteElement;

	if ('full' != options.panel) panel = cal.down('.panel');
	if (options.display) displayPanel = cal.down('.display');
	if (time) {
		hourElement = table.down('.hours');
		minuteElement = table.down('.minutes');
	}
	
	// Set events
	if ('full' == options.panel) {
		monthElement.observe('change', function() { setMonth(month()); });
		yearElement.observe('change', function() { setYear(year()); });
	}
	cal.observe('click', click);
	table.observe('mouseover', display);
	table.observe('mouseout', clean);
	
	if (options.embedded) {
		cal.style.position = target.style.position;
		try {
			cal.clonePosition(target, { setWidth: false, setHeight: false });
			if (iframe) iframe.clonePosition(cal);
		} catch(e) {}
		if ('INPUT' == target.tagName.toUpperCase()) {
			disabled = target.disabled;
			visible = target.visible();
			target.disabled = true;
			target.hide();
			inputType = new Element('input', { type: 'hidden', name: target.name });
			target.insert({ before: inputType });
		} else
			target.hide();
		cal.addClassName('embedded');
	} else {
		// Bind calendar
		if (options.popupBy != target) {
			options.popupBy.observe('click', clickPopupBy);
		} else {
			options.popupBy.observe('mousedown', show);
			cal.observe('click', function(e) { e.stop(); });
		}
		if ('text' == target.type)
			target.observe('change', setSelectedValue);
		// Close
		document.observe('mousedown', close);
	}
	
// Public methods
	function destroy() {
		if (!options.embedded) {
			options.popupBy.stopObserving('mousedown', options.popupBy != target ? clickPopupBy : show);
			target.stopObserving('change', setSelectedValue);
			document.stopObserving('mousedown', close);
		} else {
			inputType.remove();
			target.disabled = disabled;
			if (visible) target.show();
		}
		cal.remove();
	}
	
	function formattedDate() {
		return selected ? formatDate(selected.clone().resetTime()) : null;
	}
	
	function formattedDateTime() {
		return selected ? formatDate(selected.clone()) : null;
	}
	
	function hide() {
		if (options.embedded || !cal.visible()) return;
		launchEvent('beforeClose');
		cal.hide();
		if (Calendar.iframe) Calendar.iframe.hide();
		launchEvent('afterClose');
	}

	function hours() {
		return hourElement ? hourElement.selectedIndex : null;
	}

	function link(cal, exclusive) {
		if (options.embedded) return false;
		var func = function() {
			var d = cal.selectedDate();
			if (!d) return setDefaultStartEndDates();
			if (exclusive) d.add('d', -1);
			endDate = d;
		};
		if (!cal.linkedBy(that, exclusive)) return;
		observe('beforeShow', func);
		func();
		return true;
	}

	function minutes() {
		return minuteElement ? minuteElement.selectedIndex * options.minutesRange : null;
	}
	
	function month() {
		return date.getMonth();
	}

	function observe(eventName, func) {
		events[eventName] = events[eventName] || [];
		events[eventName].push(func);
	}

	function selectedDate() {
		return selected ? selected.clone().resetTime() : null;
	}

	function selectedDateTime() {
		return selected ? selected.clone() : null;
	}

	function show() {
		launchEvent('beforeShow');
		if (setSelectedValue()) {
			if (!options.fixed)
				date = selected.clone().set('d', 1);
			if (time) {
				hourElement.selectedIndex = selected.getHours();
				minuteElement.selectedIndex = (selected.getMinutes() /
					options.minutesRange).floor();
			}
		}
		refresh();
		cal.show();
		positionIt();
		launchEvent('afterShow');
	}
	
	function stopObserving(eventName, func) {
		events[eventName].splice(events[eventName].indexOf(func), 1);
	}
	
	function year() {
		return date.getFullYear();
	}

// "Protected" methods = public in javascript
// Method used for link
	function linkedBy(cal, exclusive) {
		if (options.embedded) return false;
		var func = function() {
			var d = cal.selectedDate();
			if (!d) return setDefaultStartEndDates();
			if (exclusive) d.add('d', 1);
			startDate = d;
		};
		observe('beforeShow', func);
		func();
		setSelectedValue();
		return true;
	}
	
// Private methods
	function buildCombo(arr, className, valueIndex, selectedItem) {
		return '<select size="1" class="' + className + '">\n' + 
			arr.map(function(elt, index) {
				var value = (valueIndex ? index : elt);
				return '<option ' +
					(selectedItem == value ? 'selected="selected" ' : '') +
					'value="' + value + '">' + elt + '</option>';
			}).join('') + '</select>';
	}
	
	function buildContent() {
		var d = date.getWeekDay(), klasses = ['before', '', 'after'],
			day = date.clone().add('d', -(d < 2 ? d + 7 : d)).resetTime(),
			klass, d, html = '', w, strict;
		values = [];
		for (var i = 0; i < 6; ++i) {
			html += '<tr>';
			if (options.weeks)
				html += '<th>' + ((w = day.get('w')) < 10 ? '0' + w : w) + '</th>';
			for (var j = 0; j < 7; ++j) {
				values.push(day.clone());
				d = day.getDate(), klass = '';
				if (1 == d) klasses.shift();
				strict = (options.strictMonth && klasses.length != 2) || checkDate(day);
				if (strict) klass = 'hidden ';
				if (selected && day.sameDay(selected)) klass += 'selected ';
				if (day.sameDay(today)) klass += 'today ';
				klass += (day.isWeekEnd() ? 'weekend ' : '') + klasses[0];
				klass = ('' == klass ? '' : ' class="' + klass.strip() + '"');
				html += '<td' + klass + '><a href="#" class="v' + (i * 7 + j) + '">' + d + '</a></td>';
				day.add('d', 1);
			}
			html += '</tr>';
		}
		return html;
	}
	
	function buildPanel() {
		if (!['full', 'normal', 'light'].include(options.panel)) return '';
		return '<tr class="panel">' + buildPanelContent() + '</tr>';
	}
	
	function buildPanelContent() {
		var pnl = options.panel, html = '', cols = options.weeks ? 6 : 5,
			names = Date[('short' == options.monthNames ? 'SHORT_' : '') + 'MONTHS'];
		if ('normal' == pnl || 'light' == pnl) {
			if ('light' == pnl) cols += 2;
			html += names[date.getMonth()] + ' ' + date.get('y');
		} else if ('full' == pnl) {
			var yearRange = $R(minYear, maxYear).toArray();
			html +=	buildCombo(names, 'months', true, date.getMonth()) +
				buildCombo(yearRange, 'years', false, date.getFullYear());
		}
		html = '<th colspan="' + cols + '">' + html + '</th>';
		if ('full' == pnl || 'normal' == pnl)
			html = '<th class="previous"><a href="#">&lt;&lt;</a></th>' + 
				html + '</th><th class="next"><a href="#">&gt;&gt;</a></th>';
		return html;
	}
	
	function buildTHead() {
		var days = !isNaN(parseInt(options.dayNames, 10))
				? Date.truncate_days(options.dayNames)
				: Date[('short' == options.dayNames ? 'SHORT_' : '') + 'DAYS'],
			len = days.length, html = '';
		for (var i = 0; i < len; ++i)
			html += '<th>' + days[(i + Date.firstDayOfWeek) % len] + '</th>';
		return '<thead>' + 
			buildPanel() + 
			'<tr>' + (options.weeks ? '<th>&nbsp;</th>' : '') + html + '</tr>' +
		'</thead>';
	}
	
	function buildTFoot() {
		var r = options.minutesRange, m = (date.getMinutes() / r).floor() * r,
			min = $R(0, 59).select(function(m) { return 0 == m % r }),
			cols = options.weeks ? 8 : 7, html = '', cHours = !amPm
				? $R(0, 23).toArray()
				: Date.AM_PM.inject([], function(acc, marker) {
					return $R(0, 11).inject(acc, function(acc, index) {
						index = (h && 0 == index ? 12 : index);
						acc.push((index < 10 ? '0' : '') + index + ' ' + marker);
						return acc;
					});
				});
		if (options.extraPanel)
			html = '<tr><th colspan="' + cols + '">' + 
				(!time ? ' <a href="#" class="now">' + Date.TODAY + '</a>'
				: buildCombo(cHours, 'hours', true, date.getHours()) + 
					' : ' + buildCombo(min, 'minutes', false, m) +
				' <a href="#" class="now' +
				(isNowDisabled() ? ' disabled' : '') + '">' + Date.NOW +
				'</a>') + '</th></tr>';
		if (options.display)
			html += '<tr><th colspan="' + cols + '" class="display">&nbsp;</th></tr>';
		return '' != html ? '<tfoot>' + html + '</tfoot>' : '';
	}

	function checkDate(d) {
		d = d.clone().resetTime();
	 	return startDate.isAfter(d) ? 1 : (endDate.isBefore(d) ? -1 : 0);
	}

	function click(e) {
		e.stop(); e.stopPropagation();
		var eTarget = e.target, th = eTarget.up();
		if (th && th.className == 'previous') {
			setMonth(month() - 1);
		} else if (th && th.className == 'next') {
			setMonth(month() + 1);
		} else if (eTarget.className == 'now') {
			if (isNowDisabled()) return;
			var d = new Date(), r = options.minutesRange;
			d.set('m', (d.get('m') / r).floor() * r);
			setTargetValue(formatDate(d));
			selected = d;
			launchEvent('change');
			hide();
			if (options.embedded) refresh();
		} else if (e.findElement('td')) {
			var d = getDate(eTarget);
			if (!d || eTarget.hasClassName('hidden')) return;
		  	setTargetValue(formatDate(d));
		  	selected = d;
		  	launchEvent('change');
			hide();
			var s = tbody.down('.selected');
			s && s.removeClassName('selected');
			eTarget.up().className += ' selected';
		}
	}

	function clickPopupBy() {
		cal.visible() ? hide() : show();
	}

	function clean(e) {
		if (options.display && (e.findElement('td') || e.findElement('.now')));
			displayPanel.update('&nbsp;');
	}

	function close(e) {
		if (!e.findElement('#' + id) && target != e.target &&
				options.popupBy != e.target)
			hide();
	}
	 
	function display(e) {
		var eTarget = $(e.target).up();
		if (e.findElement('.now')) {
			if (!options.display) return;
			var d = new Date(), r = options.minutesRange;
			d.set('m', (d.get('m') / r).floor() * r);
			displayPanel.update(formatDate(d, true));
		} else if ('TD' == eTarget.tagName.toUpperCase()) {
			if (eTarget.hasClassName('hidden')) return;
			if (!options.display) return;
			displayPanel.update(formatDate(getDate(e.target), true));
		}
	}

	function formatDate(d, display) {
		df.setPattern(display ? options.displayPattern : options.pattern);
		return df.format(d);
	}

	function getDate(element) {
		var d = values[element.className.substr(1, 2)];
		return time ? d.set('H', hours()).set('m', minutes()) : d;
	}

	function isNowDisabled() {
		var clone = today.clone().resetTime();
		return clone.isBefore(startDate) || clone.isAfter(endDate);
	}

	function launchEvent(eventName) {
		if (!events[eventName]) return;
		events[eventName].each(function(func) { func(that); });
	}
	
	function parseDate(s, display) {
		df.setPattern(display ? options.displayPattern : options.pattern);
		return df.parse(s);
	}

	function positionIt() {
		if (!dHeight) // IE sucks...
			tbody.up('table').style.width = cal.getWidth() + 'px';
		dHeight = dHeight || target.getHeight();
		height = height || cal.getHeight();
		var ot = (document.viewport.getHeight() - target.viewportOffset().top -
			dHeight - height <= 0) ? -height : dHeight;
		try {
			cal.clonePosition(target, { setWidth: false, setHeight: false, offsetTop: ot });
			if (!Calendar.iframe) return;
			Calendar.iframe.show();
			Calendar.iframe.clonePosition(cal);
		} catch(e) {}
	}
	
	function refresh() {
		var v = checkDate(date);
		if (0 != v)
			date = (v > 0 ? startDate : endDate).clone().set('d', 1);
		if ('full' == options.panel) {
			monthElement.selectedIndex = date.getMonth();
			yearElement.selectedIndex = date.getFullYear() - minYear;
		} else {
			panel.update(buildPanelContent());
		}
		tbody.update(buildContent());
		if (!options.extraPanel) return;
		now = now || cal.down('a.now');
		now[(isNowDisabled() ? 'add' : 'remove') + 'ClassName']('disabled');
	}

	function setDefaultDate() {
		if (0 != checkDate(date = today.clone().set('d', 1)))
			date = startDate.clone().set('d', 1);
	}

	function setDefaultStartEndDates() {
		startDate = options.startDate || (options.fixed ? new Date()
            : new Date(options.startYear, 0, 1));
		if (options.fixed) {
			options.panel = 'light';
			var d = startDate.set('d', 1).getWeekDay();
			startDate.add('d', -(d < 2 ? d + 7 : d));
		}
		endDate = options.fixed ? startDate.clone().add('d', 42)
			: (options.endDate || new Date(parseInt(options.startYear, 10)
                + parseInt(options.yearRange, 10) -1, 11, 31));
	}
	
	function setMonth(m) {
		date.setMonth(m);
		refresh();
		launchEvent('navigate');
	}

	function setSelectedValue() {
		if ('text' == target.type)
			selected = parseDate($F(target));
		if (selected && 0 == checkDate(selected)) return true;
		setTargetValue('');
		setDefaultDate();
		selected = null;
		return false;
	}

	function setTargetValue(value) {
		if (['text', 'hidden'].include(target.type))
			target.setValue(value);
	}
	
	function setYear(y) {
		date.setFullYear(y);
		refresh();
		launchEvent('navigate');
	}
	
	function setTime(h, m) {
		date.set('H', h).set('m', h);
		refresh();
	}
	
	// ie6 sucks...
	if (Prototype.Browser.IE && Calendar.first && !options.embedded) {
		Calendar.first = false; positionIt();
	}
	
	var that = {
		destroy: destroy,
		formattedDate: formattedDate,
		formattedDateTime: formattedDateTime,
		hide: hide,
		hours: hours,
		link: link,
		minutes: minutes,
		month: month,
		observe: observe,
		selectedDate: selectedDate,
		selectedDateTime: selectedDateTime,
		show: show,
		stopObserving: stopObserving,
		year: year,
	// protected method
		linkedBy: linkedBy
	};
	return that;
};

Object.extend(Calendar, {
  id: 0,
  first: true,
  defaultCss: true,
  dateFormat: new DateFormat(/ /),
  bind: function(expressions, options) {
  	return $$(expressions).inject([], function(array, element) {
		array.push(new Calendar(element, options));
		return array;
  	});
  },
  addDefaultCss: function() {
  	if (!this.defaultCss) return;
  	Css.addRules("\
.calendar { -moz-border-radius: 6px; -o-border-radius: 6px; border-radius: 6px; text-align: left; }\n\
.calendar { font: normal 11px verdana, arial, helvetica, sans-serif; }\n\
.calendar { background-color: #efefef; padding: 10px; border: 1px solid #969696; }\n\
.calendar table { border-collapse: collapse; text-align: center; clear: both; }\n\
.calendar td { border: 1px solid #d4d4d4; }\n\
.calendar td a { border: 1px solid white; display: block; text-decoration: none; color: #1c54c9; }\n\
.calendar .panel th { font-weight: bold; padding-bottom: 0.6em; text-align: center; }\n\
.calendar .panel a { text-decoration: none; font-weight: bold; }\n\
.calendar a.now.disabled { color: gray; }\n\
.calendar a.now { margin-left: 1em; }\n\
.calendar .previous { text-align: left; padding-left: 0; }\n\
.calendar .next { text-align: right; padding-right: 0; }\n\
.calendar tbody td { font-weight: bold; background-color: white; }\n\
.calendar td a, .calendar th { padding: 0.25em; }\n\
.calendar table, .calendar tbody th, .calendar .hidden { cursor: default; }\n\
/*.calendar .weekend { background-color: #ddd; }*/\n\
.calendar .today a { border: 1px solid black; }\n\
.calendar .selected a { background-color: #bbf; }\n\
.calendar .before, .calendar .after { border: 1px solid #d1d1d1; }\n\
.calendar .before a, .calendar .after a { background-color: #d1d1d1; border-color: #d1d1d1; color: #9697a0; }\n\
.calendar td a:hover { background-color: #1c54c9; color: white; }\n\
.calendar tbody .hidden a { visibility: hidden; cursor: default; }", 'screen', false);
  	this.defaultCss = false;
  }
});
